Package: vdiffr
Title: Visual Regression Testing and Graphical Diffing
Version: 1.0.9
Authors@R: c(
    person("Lionel", "Henry", , "lionel@posit.co", role = "aut"),
    person("Thomas Lin", "Pedersen", , "thomas.pedersen@posit.co", role = c("cre", "aut"),
           comment = c(ORCID = "0000-0002-5147-4711")),
    person("Posit Software, PBC", role = c("cph", "fnd")),
    person("T Jake", "Luciani", , "jake@apache.org", role = "aut",
           comment = "svglite"),
    person("Matthieu", "Decorde", , "matthieu.decorde@ens-lyon.fr", role = "aut",
           comment = "svglite"),
    person("Vaudor", "Lise", , "lise.vaudor@ens-lyon.fr", role = "aut",
           comment = "svglite"),
    person("Tony", "Plate", role = "ctb",
           comment = "svglite: Early line dashing code"),
    person("David", "Gohel", role = "ctb",
           comment = "svglite: Line dashing code and raster code"),
    person("Yixuan", "Qiu", role = "ctb",
           comment = "svglite: Improved styles; polypath implementation"),
    person("Håkon", "Malmedal", role = "ctb",
           comment = "svglite: Opacity code")
  )
Description: An extension to the 'testthat' package that makes it easy to
    add graphical unit tests. It provides a Shiny application to manage
    the test cases.
License: MIT + file LICENSE
URL: https://vdiffr.r-lib.org/, https://github.com/r-lib/vdiffr
BugReports: https://github.com/r-lib/vdiffr/issues
Depends: R (>= 4.0)
Imports: diffobj, glue, grDevices, htmltools, lifecycle, rlang,
        testthat (>= 3.0.3), xml2 (>= 1.0.0)
Suggests: covr, decor, ggplot2 (>= 3.2.0), roxygen2, withr
LinkingTo: cpp11
ByteCompile: true
Config/Needs/website: tidyverse/tidytemplate
Encoding: UTF-8
RoxygenNote: 7.3.2
SystemRequirements: libpng
NeedsCompilation: yes
Packaged: 2026-02-12 12:25:53 UTC; lionel
Author: Lionel Henry [aut],
  Thomas Lin Pedersen [cre, aut] (ORCID:
    <https://orcid.org/0000-0002-5147-4711>),
  Posit Software, PBC [cph, fnd],
  T Jake Luciani [aut] (svglite),
  Matthieu Decorde [aut] (svglite),
  Vaudor Lise [aut] (svglite),
  Tony Plate [ctb] (svglite: Early line dashing code),
  David Gohel [ctb] (svglite: Line dashing code and raster code),
  Yixuan Qiu [ctb] (svglite: Improved styles; polypath implementation),
  Håkon Malmedal [ctb] (svglite: Opacity code)
Maintainer: Thomas Lin Pedersen <thomas.pedersen@posit.co>
Repository: CRAN
Date/Publication: 2026-02-13 08:20:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-25 18:40:14 UTC; unix
Archs: vdiffr.so.dSYM
