# Generated by cpp11: do not edit by hand

compare_files <- function(expected, test) {
  .Call(`_vdiffr_compare_files`, expected, test)
}

svglite_ <- function(file, bg, width, height, pointsize, standalone, always_valid) {
  .Call(`_vdiffr_svglite_`, file, bg, width, height, pointsize, standalone, always_valid)
}

svgstring_ <- function(env, bg, width, height, pointsize, standalone) {
  .Call(`_vdiffr_svgstring_`, env, bg, width, height, pointsize, standalone)
}

get_svg_content <- function(p) {
  .Call(`_vdiffr_get_svg_content`, p)
}

set_engine_version <- function(version) {
  invisible(.Call(`_vdiffr_set_engine_version`, version))
}
