# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

compare_files <- function(expected, test) {
    .Call(`_vdiffr_compare_files`, expected, test)
}

svglite_ <- function(file, bg, width, height, pointsize, standalone, aliases) {
    .Call(`_vdiffr_svglite_`, file, bg, width, height, pointsize, standalone, aliases)
}

svgstring_ <- function(env, bg, width, height, pointsize, standalone, aliases) {
    .Call(`_vdiffr_svgstring_`, env, bg, width, height, pointsize, standalone, aliases)
}

get_svg_content <- function(p) {
    .Call(`_vdiffr_get_svg_content`, p)
}

library_load <- function() {
    .Call(`_vdiffr_library_load`)
}

test_string_width <- function(string, font_size, font_file) {
    .Call(`_vdiffr_test_string_width`, string, font_size, font_file)
}

test_string_info <- function(string, font_size, font_file) {
    .Call(`_vdiffr_test_string_info`, string, font_size, font_file)
}

